from pathlib import Path
import re
from hivebox.common import cli_print, cli_run, PrintColors

interfaces_config = """auto {interface_name}
iface {interface_name} inet static
        address 192.168.192.1
        netmask 255.255.255.0
        dns-nameservers 8.8.8.8"""

networkmanager_config = """[main]
plugins=ifupdown,keyfile

[ifupdown]
managed=true

[device]
wifi.scan-rand-mac-address=no"""

dnsmasq_config = """domain=hivecv.local
no-resolv
no-poll
dhcp-range=192.168.192.101,192.168.192.120,255.255.255.0,12h
dhcp-option=3,192.168.192.1
dhcp-option=6,8.8.8.8
log-queries
log-dhcp"""

class DHCPSetup:
    @staticmethod
    def _ensure_installed(pkg_name):
        result = cli_run(f"dpkg -s {pkg_name}", stdout=True, silent=True)
        if result.returncode == 0:
            m = re.search("Status:.*", result.stdout)
            if m:
                installation_status = m.group()[len("Status:"):].strip().split()
                if 'installed' in installation_status and 'ok' in installation_status:
                    cli_print(f"{pkg_name} already installed")
                    return

        cli_print(f"Installing {pkg_name}...", PrintColors.BLUE)
        cli_run(f"apt-get install -y {pkg_name}")
        cli_print(f"{pkg_name} successfully installed")

    @staticmethod
    def install_rpi5():
        cli_print("Starting DHCP installation...")
        DHCPSetup._ensure_installed("dnsmasq")

        cli_print("Configuring Static IP...")
        with Path("/etc/network/interfaces").open(mode="w") as f:
            f.write(interfaces_config.format(interface_name="eth0"))
        cli_print("Configuring Network Manager...")
        with Path("/etc/NetworkManager/NetworkManager.conf").open(mode="w") as f:
            f.write(networkmanager_config)
        cli_print("Configuring DHCP Server...")
        with Path("/etc/dnsmasq.conf").open(mode="w") as f:
            f.write(dnsmasq_config)

        cli_run("systemctl restart NetworkManager", label="NetworkManager restart")
        cli_run("systemctl restart dnsmasq", label="dnsmasq restart")

        cli_print("Finished DHCP installation!")

    @staticmethod
    def install_orangepi5b():
        cli_print("Starting DHCP installation...")
        DHCPSetup._ensure_installed("dnsmasq")

        cli_print("Configuring Static IP...")
        with Path("/etc/network/interfaces").open(mode="w") as f:
            f.write(interfaces_config.format(interface_name="end1"))
        cli_print("Configuring DHCP Server...")
        with Path("/etc/dnsmasq.conf").open(mode="w") as f:
            f.write(dnsmasq_config)

        cli_run("ip addr flush dev end1", label="Previous IP config removal")
        cli_run("systemctl restart networking", label="networking restart")
        cli_run("systemctl restart dnsmasq", label="dnsmasq restart")

        cli_print("Finished DHCP installation!")




import threading
from pathlib import Path, PurePosixPath
from hivebox.networking.utils import get_ip
from flask import Flask, Blueprint, redirect
from werkzeug.serving import make_server
from hivebox.networking.index_module import Module as IndexModule
from urllib.parse import unquote, urlparse



class WebServer(threading.Thread):
    def __init__(self):
        super().__init__()
        self.running = threading.Event()
        self.modules = {}
        self.blueprints = {}
        self.index_module = IndexModule(modules=self.modules)
        self.register_module(self.index_module)
        self.app = Flask(__name__)

        @self.app.route("/")
        def home():
            return redirect(self.index_module.Meta.base_url)

        self.server = make_server('0.0.0.0', 5001, self.app, threaded=True)
        self.ctx = self.app.app_context()
        self.ctx.push()

    # NOTE: Use start() to start the thread, this method is a new thread starting point
    def run(self):
        for module_id, blueprint in self.blueprints.items():
            module = self.modules[module_id]
            print(f"Enabling \"{module.Meta.module_name}\" module...")
            self.app.register_blueprint(blueprint, url_prefix=module.Meta.base_url)
        print('WebServer starting...')
        self.running.set()
        print("WebServer started at http://{}:{}".format(get_ip(self.server.host), self.server.port))

        self.server.serve_forever()

    def stop(self):
        print('WebServer stopping...')
        self.running.clear()
        self.server.shutdown()

    def register_module(self, module):
        if not hasattr(module, 'Meta'):
            raise RuntimeError('Module "{}" has no Meta class!'.format(module))

        if not hasattr(module.Meta, 'module_id'):
            raise RuntimeError('Module "{}" Meta class has no module_id!'.format(module))

        if module.Meta.module_id in self.modules:
            raise RuntimeError(f'Module with ID "{module.Meta.module_id}" already registered')

        if not hasattr(module.Meta, 'base_url'):
            raise RuntimeError('Module "{}" Meta class has no module_id!'.format(module))

        if module.Meta.base_url in ('', None, '/'):
            raise RuntimeError('Module "{}" Meta class has incorrect base_url!'.format(module))

        if len(PurePosixPath(unquote(urlparse(module.Meta.base_url).path)).parts) < 2 and module.Meta.module_id != IndexModule.Meta.module_id:
            raise RuntimeError(f'Module with ID "{module.Meta.module_id}" has invalid base_url value')

        self.blueprints[module.Meta.module_id] = Blueprint(
            name=module.Meta.module_id,
            import_name=__name__,
            static_folder=str(Path(module.Meta.static_path).absolute()) if getattr(module.Meta, 'static_path', None) else None,
            template_folder=str(Path(module.Meta.template_path).absolute()) if getattr(module.Meta, 'template_path', None) else None,
        )
        module.enable(self.blueprints[module.Meta.module_id])
        self.modules[module.Meta.module_id] = module


if __name__ == "__main__":
    server = WebServer()
    server.start()
    try:
        while True:
            pass
    finally:
        server.stop()



import time
import cv2
from hivebox.common.cli import cli_print, PrintColors
import depthai as dai
import numpy as np


def wait_for_device(mxid, timeout=30, interval=0.1):
    start = time.time()
    while time.time() - start < timeout:
        devices = list(map(lambda info: info.getMxId(), dai.XLinkConnection.getAllConnectedDevices()))
        if mxid in devices:
            return True
        else:
            time.sleep(interval)
    else:
        cli_print(f"[Warning] Device not available again after 30 seconds! MXID: {mxid}", PrintColors.RED)
        return False


def select_device():
    infos = dai.XLinkConnection.getAllConnectedDevices()

    if len(infos) == 0:
        raise RuntimeError("No DepthAI device found!")
    else:
        print("Available devices:")
        for i, info in enumerate(infos):
            print(f"[{i}] {info.getMxId()} [{info.state.name}]")

        if len(infos) == 1:
            return infos[0]
        else:
            val = input("Which DepthAI Device you want to use: ")
            try:
                return infos[int(val)]
            except:
                raise ValueError("Incorrect value supplied: {}".format(val))


def to_planar(arr: np.ndarray, shape: tuple = None) -> np.ndarray:
    if shape:
        return cv2.resize(arr, shape).transpose(2, 0, 1).flatten()
    else:
        return arr.transpose(2, 0, 1).flatten()

def frame_norm(frame, bbox):
    normVals = np.full(len(bbox), frame.shape[0])
    normVals[::2] = frame.shape[1]
    return (np.clip(np.array(bbox), 0, 1) * normVals).astype(int)



import argparse

from hivebox.networking.dhcp import DHCPSetup


def main():
    parser = argparse.ArgumentParser(description='Install DHCP server')
    parser.add_argument('--rpi5', help='Install on RaspberryPi 5', action='store_true')
    parser.add_argument('--orangepi5b', help='Install on OrangePi 5b', action='store_true')
    args = vars(parser.parse_args())

    if args.get('rpi5'):
        DHCPSetup.install_rpi5()
    elif args.get('orangepi5b'):
        DHCPSetup.install_orangepi5b()
    else:
        raise RuntimeError('You must specify either --rpi5 or --orangepi5b')

if __name__ == '__main__':
    main()


import psutil
import socket

def get_ip(default='127.0.0.1'):
    tailscale_ip = next(map(
        lambda item: item.address,
        filter(
            lambda item: item.family == socket.AF_INET,
            psutil.net_if_addrs().get('tailscale0', [])
        )
    ), None)
    if tailscale_ip is not None:
        return tailscale_ip

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(0)
    try:
        s.connect(('8.8.8.8', 1))
        local_ip = s.getsockname()[0]
    except:
        local_ip = default
    finally:
        s.close()
    return local_ip


import sys
from pathlib import Path
import re
from hivebox.common import cli_print, cli_run, PrintColors

class GPIOSetup:
    @staticmethod
    def install_orangepi5b(library_path: Path):
        wiringOP_path = library_path / "wiringOP"
        cli_print("Starting GPIO installation...")
        cli_print("Cloning repository...")
        cli_run(f"git clone --recursive https://github.com/orangepi-xunlong/wiringOP-Python -b next {wiringOP_path}", label="Downloading WiringOp...")
        print(cli_run("pwd", workdir=wiringOP_path, stdout=True).stdout)
        cli_run(f"git submodule update --init --remote", workdir=wiringOP_path)
        result = cli_run(f"{sys.executable} generate-bindings.py", workdir=wiringOP_path, stdout=True)
        with wiringOP_path.joinpath('bindings.i').open('w') as f:
            f.write(result.stdout)
        cli_run(f"{sys.executable} setup.py install", workdir=wiringOP_path)
        cli_print("Installation complete")
        print(cli_run("gpio readall", workdir=wiringOP_path, stdout=True).stdout)
        import wiringpi  # NOQA



import sqlite3
from pathlib import Path


class SQLite:
    def __init__(self, create_sql, db_name):
        self.db_name = db_name

        if not Path(self.db_name).exists():
            with sqlite3.connect(self.db_name) as conn:
                conn.execute(create_sql)

    def get_data(self, table_name, order_by=None):
        with sqlite3.connect(self.db_name) as conn:
            query = f"SELECT * FROM {table_name}"
            if order_by:
                ascending = True
                if order_by.startswith("-"):
                    order_by = order_by[1:]
                    ascending = False
                query += f" ORDER BY {order_by} {'ASC' if ascending else 'DESC'}"
            print(query)
            result = conn.execute(query)
            colnames = [d[0] for d in result.description]
            return [dict(zip(colnames, result_list)) for result_list in result.fetchall()]

    def add_data(self, table_name, input_data: list[dict]):
        if len(input_data) == 0:
            return
        cols = input_data[0].keys()
        data = [tuple([item[col] for col in cols]) for item in input_data]
        with sqlite3.connect(self.db_name) as conn:
            sql = f"INSERT INTO {table_name} ({','.join(cols)}) VALUES ({','.join('?' * len(cols))})"
            return conn.executemany(sql, data)

if __name__ == "__main__":
    db = SQLite("""
    CREATE TABLE people (
                        firstname varchar,
                        surname varchar,
                        age numeric,
                        timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL
                    );
    """)
    db.add_data('people', [
        {'firstname': 'Jakub', 'surname': 'Kowalski', 'age': 33},
        {'firstname': 'Łukasz', 'surname': 'Piłatowski', 'age': 28},
    ])

    print(db.get_data('people'))


import socketserver
import threading
from http.server import BaseHTTPRequestHandler, HTTPServer
from socketserver import ThreadingMixIn
from time import sleep

import cv2
import depthai as dai

HTTP_SERVER_PORT = 8090


class TCPServerRequest(socketserver.BaseRequestHandler):
    def handle(self):
        header = 'HTTP/1.0 200 OK\r\nServer: Mozarella/2.2\r\nAccept-Range: bytes\r\nConnection: close\r\nMax-Age: 0\r\nExpires: 0\r\nCache-Control: no-cache, private\r\nPragma: no-cache\r\nContent-Type: application/json\r\n\r\n'
        self.request.send(header.encode())
        while True:
            sleep(0.1)
            if hasattr(self.server, 'datatosend'):
                self.request.send(self.server.datatosend.encode() + "\r\n".encode())


class VideoStreamHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        try:
            self.send_response(200)
            self.send_header('Content-type', 'multipart/x-mixed-replace; boundary=--jpgboundary')
            self.end_headers()
            while True:
                sleep(0.1)
                if hasattr(self.server, 'frametosend'):
                    ok, encoded = cv2.imencode('.jpg', self.server.frametosend)
                    self.wfile.write("--jpgboundary".encode())
                    self.send_header('Content-type', 'image/jpeg')
                    self.send_header('Content-length', str(len(encoded)))
                    self.end_headers()
                    self.wfile.write(encoded)
                    self.end_headers()
        except BrokenPipeError:
            print("Broken pipe, closing connection...")


class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
    """Handle requests in a separate thread."""
    pass


def get_ip():
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(0)
    try:
        s.connect(('8.8.8.8', 1))
        ip = s.getsockname()[0]
    except Exception:
        ip = '127.0.0.1'
    finally:
        s.close()
    return ip


def main():
    print("Running Hivebox Live...")

    server_HTTP = ThreadedHTTPServer(('0.0.0.0', HTTP_SERVER_PORT), VideoStreamHandler)
    th2 = threading.Thread(target=server_HTTP.serve_forever)
    th2.daemon = True
    th2.start()

    def create_pipeline():
        pipeline = dai.Pipeline()
        cam = pipeline.create(dai.node.ColorCamera)
        cam.setPreviewSize(300, 300)
        cam.setResolution(dai.ColorCameraProperties.SensorResolution.THE_1080_P)
        cam.setInterleaved(False)

        xoutRgb = pipeline.create(dai.node.XLinkOut)
        xoutRgb.setStreamName("rgb")
        cam.preview.link(xoutRgb.input)
        return pipeline

    # Pipeline is defined, now we can connect to the device
    with dai.Device(create_pipeline()) as device:
        print("DepthAI is up & running.")
        print(f"Navigate to 'http://{get_ip()}:{str(HTTP_SERVER_PORT)}' with Chrome to see the mjpeg stream")

        previewQueue = device.getOutputQueue(name="rgb", maxSize=4, blocking=False)

        while True:
            server_HTTP.frametosend = previewQueue.get().getCvFrame()

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import threading
import traceback
from queue import Queue
from threading import Event

import cv2
import depthai as dai
import numpy as np

from hivebox.common import ConfigWrapper, select_device, wait_for_device

class VideoSaver(threading.Thread):
    _worker_q = Queue(maxsize=10)
    _running = Event()

    def __init__(self, output_path: str):
        super().__init__()
        self._output_path = output_path

    # NOTE: Use start() to start the thread, this method is a new thread starting point
    def run(self):
        self._running.set()
        print("Starting saver...")
        writer = None
        while self._running.is_set():
            try:
                if not self._worker_q.empty():
                    frame = self._worker_q.get()
                    if not writer:
                        writer = cv2.VideoWriter(self._output_path, cv2.VideoWriter_fourcc(*"mp4v"), 30, (frame.shape[1], frame.shape[0]))
                    writer.write(frame)
            except:
                traceback.print_exc()
                break
        print("Terminating saver...")
        if writer:
            writer.release()

    def store_frame(self, frame):
        self._worker_q.put(frame)

    def stop(self):
        print('Saver stopping...')
        self._running.clear()

def create_pipeline(rgb_res, lrc, ext, sub):
    pipeline = dai.Pipeline()

    rgb = pipeline.create(dai.node.ColorCamera)
    left = pipeline.create(dai.node.MonoCamera)
    right = pipeline.create(dai.node.MonoCamera)
    depth = pipeline.create(dai.node.StereoDepth)
    xout_disp = pipeline.create(dai.node.XLinkOut)
    xout_rgb = pipeline.create(dai.node.XLinkOut)

    rgb.setInterleaved(False)
    rgb.setColorOrder(dai.ColorCameraProperties.ColorOrder.RGB)
    rgb.setResolution(rgb_res)
    rgb.setVideoNumFramesPool(10)

    xout_disp.setStreamName("disparity")
    xout_rgb.setStreamName("color")

    left.setResolution(dai.MonoCameraProperties.SensorResolution.THE_800_P)
    left.setCamera("left")
    right.setResolution(dai.MonoCameraProperties.SensorResolution.THE_800_P)
    right.setCamera("right")

    depth.initialConfig.setMedianFilter(dai.MedianFilter.KERNEL_3x3)
    depth.setDepthAlign(dai.CameraBoardSocket.CAM_A)
    depth.setLeftRightCheck(lrc)
    depth.setExtendedDisparity(ext)
    depth.setSubpixel(sub)

    # Script node will sync high-res frames
    script = pipeline.create(dai.node.Script)

    # Send both streams to the Script node so we can sync them
    depth.disparity.link(script.inputs["disp_in"])
    rgb.video.link(script.inputs["rgb_in"])

    script.setScript("""
        FPS=30
        import time
        from datetime import timedelta
        import math

        # Timestamp threshold (in miliseconds) under which frames will be considered synced.
        # Lower number means frames will have less delay between them, which can potentially
        # lead to dropped frames.
        MS_THRESHOL=math.ceil(500 / FPS)

        def check_sync(queues, timestamp):
            matching_frames = []
            for name, list in queues.items(): # Go through each available stream
                # node.warn(f"List {name}, len {str(len(list))}")
                for i, msg in enumerate(list): # Go through each frame of this stream
                    time_diff = abs(msg.getTimestamp() - timestamp)
                    if time_diff <= timedelta(milliseconds=MS_THRESHOL): # If time diff is below threshold, this frame is considered in-sync
                        matching_frames.append(i) # Append the position of the synced frame, so we can later remove all older frames
                        break

            if len(matching_frames) == len(queues):
                # We have all frames synced. Remove the excess ones
                i = 0
                for name, list in queues.items():
                    queues[name] = queues[name][matching_frames[i]:] # Remove older (excess) frames
                    i+=1
                return True
            else:
                return False # We don't have synced frames yet

        names = ['disp', 'rgb']
        frames = dict() # Dict where we store all received frames
        for name in names:
            frames[name] = []

        while True:
            for name in names:
                f = node.io[name+"_in"].tryGet()
                if f is not None:
                    frames[name].append(f) # Save received frame

                    if check_sync(frames, f.getTimestamp()): # Check if we have any synced frames
                        # Frames synced!
                        node.info(f"Synced frame!")
                        # node.warn(f"Queue size. Disp: {len(frames['disp'])}, rgb: {len(frames['rgb'])}")
                        for name, list in frames.items():
                            syncedF = list.pop(0) # We have removed older (excess) frames, so at positions 0 in dict we have synced frames
                            node.info(f"{name}, ts: {str(syncedF.getTimestamp())}, seq {str(syncedF.getSequenceNum())}")
                            node.io[name+'_out'].send(syncedF) # Send synced frames to the host


            time.sleep(0.001)  # Avoid lazy looping
    """)

    script.outputs['disp_out'].link(xout_disp.input)
    script.outputs['rgb_out'].link(xout_rgb.input)

    # Linking
    left.out.link(depth.left)
    right.out.link(depth.right)

    return pipeline


def main():
    config = ConfigWrapper()
    config.check_udev_rules()

    device_info = select_device()
    with dai.Device(device_info) as device:
        usb_2 = config.is_usb2(device)
        resolution = config.get_resolution(device)
        ir_enabled = config.ir_enabled(device)

    found = wait_for_device(device_info.mxid)

    if not found:
        raise RuntimeError("Device not found after reboot!")
    pipeline = create_pipeline(
        rgb_res=resolution,
        lrc=config.args.stereoLrCheck,
        ext=config.args.extendedDisparity,
        sub=config.args.subpixel,
    )

    saver = VideoSaver('out.mp4')
    saver.start()

    with dai.Device(pipeline, maxUsbSpeed=dai.UsbSpeed.HIGH if usb_2 else dai.UsbSpeed.SUPER_PLUS) as device:
        if ir_enabled:
            device.setIrLaserDotProjectorIntensity(1)

        q_disp = device.getOutputQueue(name="disparity", maxSize=4, blocking=False)
        q_rgb = device.getOutputQueue(name="color", maxSize=4, blocking=False)
        while True:
            try:

                rgb_packet = q_rgb.get()
                disp_packet = q_disp.get()
                rgb_frame = rgb_packet.getCvFrame()
                disp_frame = disp_packet.getCvFrame()
                disp_frame = (disp_frame * (255 / config.max_disparity)).astype(np.uint8)
                disp_frame = cv2.applyColorMap(disp_frame, cv2.COLORMAP_JET)
                combined = np.concatenate((rgb_frame, disp_frame), axis=0)
                saver.store_frame(combined)
            except KeyboardInterrupt:
                print("Terminating...")
                saver.stop()
                break

if __name__ == '__main__':
    main()



from datetime import datetime, UTC
from pathlib import Path
from hivebox.storage import SQLite
from hivebox.common.time import human_time_duration
from flask import render_template, Response
import numpy as np
import cv2

_empty_frame = np.zeros((300, 300, 3), np.uint8)

class Module:
    class Meta:
        base_url = '/home'
        module_id = 'index'
        module_name = 'Home Page'
        template_path = Path(__file__).parent / 'templates'
        static_path = Path(__file__).parent / 'static'

    def __init__(self, modules):
        self._modules = modules

    _frame = _empty_frame
    _db = SQLite(
        create_sql="""
            CREATE TABLE stats (
                cpu numeric,
                ram numeric,
                disk numeric,
                app_uptime timestamp,
                device_uptime timestamp,
                os_name text,
                os_version text,
                python_version text,
                timestamp timestamp default current_timestamp not null
            );
        """,
        db_name='index.db'
    )

    def enable(self, app):
        @app.route("/")
        def index():
            stats = self._db.get_data('stats', order_by='-timestamp')

            def chart_mapping(field):
                def result_mapping(item):
                    return (
                        human_time_duration((datetime.now(UTC) - datetime.strptime(item['timestamp'], "%Y-%m-%d %H:%M:%S").replace(tzinfo=UTC)).total_seconds()),
                        item[field]
                    )
                mapped = list(map(result_mapping, stats))
                factor = len(mapped) // 6
                result = []
                for i in range(6):
                    idx = i * factor
                    idx_next = (i + 1) * factor
                    label = mapped[idx][0]
                    slice = mapped[idx:(idx_next - 1)]
                    value = round(sum(map(lambda x: x[1], slice)) / len(slice), 2)
                    result.append((label, value))
                return result

            data = {
                'app_uptime': human_time_duration(next(map(lambda x: x['app_uptime'], stats))),
                'device_uptime': human_time_duration(next(map(lambda x: x['device_uptime'], stats))),
                'os_name': next(map(lambda x: x['os_name'], stats)),
                'os_version': next(map(lambda x: x['os_version'], stats)),
                'python_version': next(map(lambda x: x['python_version'], stats)),
                'cpu': chart_mapping('cpu'),
                'ram': chart_mapping('ram'),
                'disk': next(map(lambda x: x['disk'], stats)),
                'modules': list(filter(
                    lambda module: module.module_id != self.Meta.module_id,
                    map(lambda module: module.Meta, self._modules.values())
                ))
            }

            return render_template('index.html', **data)

        @app.route('/video_feed')
        def video_feed():
            """Video streaming route. Put this in the src attribute of an img tag."""
            return Response(self.gen(), mimetype='multipart/x-mixed-replace; boundary=frame')


    def set_frame(self, frame: np.ndarray):
        self._frame = frame

    def clear(self):
        self._frame = _empty_frame

    def gen(self):
        """Video streaming generator function."""
        while True:
            if self._frame is not None:
                yield (b'--frame\r\n'
                       b'Content-Type: image/jpeg\r\n\r\n' + bytes(cv2.imencode('.jpg', self._frame)[1]) + b'\r\n')

    def set_stats(self, cpu, ram, disk, app_uptime, device_uptime, os_name, os_version, python_version):
        self._db.add_data('stats', [{
            'cpu': cpu,
            'ram': ram,
            'disk': disk,
            'app_uptime': app_uptime,
            'device_uptime': device_uptime,
            'os_name': os_name,
            'os_version': os_version,
            'python_version': python_version,
        }])


import argparse

from hivebox.networking.web import WebServer


def main():
    parser = argparse.ArgumentParser(description='Run web server')
    args = vars(parser.parse_args())

    server = WebServer()
    server.start()
    try:
        while True:
            pass
    finally:
        server.stop()

if __name__ == '__main__':
    main()


import contextlib
import csv
import platform
import sys
import threading
import time
from pathlib import Path

import psutil
import shutil


class Timer:
    def __init__(self, output=None):
        self._results = []
        self._output = output

    @contextlib.contextmanager
    def measure(self, name):
        start = time.time()
        yield
        end = time.time()

        self._results.append((end, name, end - start))

    def dump(self):
        if self._output is None:
            print(self._results)
            return

        with open(self._output, 'w') as f:
            writer = csv.writer(f)
            writer.writerow(['time', 'name', 'elapsed'])
            for item in self._results:
                writer.writerow(item)


class PerformanceWatcher(threading.Thread):
    def __init__(self, on_report=None, interval=5):
        super().__init__()
        self.running = threading.Event()
        self.on_report = on_report
        self.interval = interval

    # NOTE: Use start() to start the thread, this method is a new thread starting point
    def run(self):
        print('PerformanceWatcher starting...')
        self.running.set()
        print("PerformanceWatcher started.")
        last_report_time = None
        while self.running.is_set():
            if last_report_time is None or (time.time() - last_report_time) > self.interval:
                disk_total, disk_used, disk_free = shutil.disk_usage(Path(__file__).absolute())
                report = {
                    "cpu": psutil.cpu_percent(),
                    "ram": psutil.virtual_memory().percent,
                    "disk": disk_used / disk_total,
                    "app_uptime": time.process_time(),
                    "device_uptime": time.time() - psutil.boot_time(),
                    "os_name": platform.system(),
                    "os_version": platform.release(),
                    "python_version": sys.version
                }
                if self.on_report is not None:
                    self.on_report(report)
                last_report_time = time.time()
            else:
                time.sleep(1)

    def stop(self):
        print('PerformanceWatcher stopping...')
        self.running.clear()

if __name__ == '__main__':
    pw = PerformanceWatcher()
    pw.start()
    start = time.time()
    time.sleep(5)
    pw.stop()


import argparse
from pathlib import Path
from hivebox.hardware.gpio import  GPIOSetup

path = Path(__file__).resolve().parent
print(path)

def main():
    parser = argparse.ArgumentParser(description='Install GPIO interface')
    parser.add_argument('--orangepi5b',  help='Install on OrangePi 5b', action='store_true')
    args = vars(parser.parse_args())

    if args.get('orangepi5b'):
        GPIOSetup.install_orangepi5b(library_path=path)
    else:
        raise RuntimeError('You must specify --orangepi5b')

if __name__ == '__main__':
    main()


TIME_DURATION_UNITS = (
    ('week', 60*60*24*7),
    ('day', 60*60*24),
    ('hour', 60*60),
    ('min', 60),
    ('sec', 1)
)


def human_time_duration(seconds, full=False):
    if seconds == 0:
        return 'inf'
    parts = []
    for unit, div in TIME_DURATION_UNITS:
        amount, seconds = divmod(int(seconds), div)
        if amount > 0:
            parts.append('{} {}{}'.format(amount, unit, "" if amount == 1 else "s"))
    return ', '.join(parts) if full else parts[0]


import argparse
import platform
import subprocess
from pathlib import Path
from typing import Union
import shlex
from subprocess import run, PIPE, DEVNULL, CompletedProcess
import depthai as dai

class PrintColors:
    HEADER="\033[95m"
    BLUE="\033[94m"
    GREEN="\033[92m"
    RED="\033[91m"
    WARNING="\033[1;5;31m"
    FAIL="\033[91m"
    ENDC="\033[0m"
    BOLD="\033[1m"
    UNDERLINE="\033[4m"
    BLACK_BG_RED="\033[1;31;40m"
    BLACK_BG_GREEN="\033[1;32;40m"
    BLACK_BG_BLUE="\033[1;34;40m"


def cli_print(msg, print_color=PrintColors.GREEN):
    print("{0}{1}{2}".format(print_color, msg, PrintColors.ENDC))


def cli_run(cmd: Union[str, list], stdout: bool = False, label: str = None, silent: bool = False, workdir: Path | str = None) -> CompletedProcess[str]:
    if label is None:
        label = " ".join(cmd) if isinstance(cmd, list) else cmd
    cli_print(f"Executing: \"{label}\"")
    result = run(cmd if isinstance(cmd, list) else shlex.split(cmd), stdout=PIPE if stdout else DEVNULL, stderr=PIPE, universal_newlines=True, cwd=str(Path(workdir).absolute()) if workdir else None)
    if result.returncode != 0 and not silent:
        cli_print(f"Error while executing \"{label}\": {result.stderr}", PrintColors.FAIL)
        raise SystemExit(1)
    return result


def _check_range(min_v, max_v):
    def check(raw_value):
        value = int(raw_value)
        if min_v <= value <= max_v:
            return value
        else:
            raise argparse.ArgumentTypeError(
                "{} is an invalid int value, must be in range {}..{}".format(raw_value, min_v, max_v)
            )

    return check


class ConfigWrapper:
    def __init__(self):
        parser = argparse.ArgumentParser()

        parser.add_argument('-dd', '--disableDepth', action="store_true", help="Disable depth information")
        parser.add_argument('--sharpness', default=None, type=_check_range(0, 4),
                            help="Sets ColorCamera's sharpness")
        parser.add_argument('--lumaDenoise', default=None, type=_check_range(0, 4),
                            help="Sets ColorCamera's Luma denoise")
        parser.add_argument('--chromaDenoise', default=None, type=_check_range(0, 4),
                            help="Sets ColorCamera's Chroma denoise")
        parser.add_argument('--manualFocus', default=None, type=_check_range(0, 255),
                            help="Specify a Lens Position between 0 and 255 to use manual focus. Otherwise, auto-focus is used by default.")
        parser.add_argument("--cameraExposure", default=None, type=int, help="Specify camera saturation")
        parser.add_argument("--cameraSensitivity", default=None, type=int, help="Specify camera sensitivity")
        parser.add_argument("--cameraSaturation", default=None, type=int, help="Specify image saturation")
        parser.add_argument("--cameraContrast", default=None, type=int, help="Specify image contrast")
        parser.add_argument("--cameraBrightness", default=None, type=int, help="Specify image brightness")
        parser.add_argument("--cameraSharpness", default=None, type=int, help="Specify image sharpness")
        parser.add_argument('-dlrc', '--disableStereoLrCheck', action="store_false", dest="stereoLrCheck",
                            help="Disable stereo 'Left-Right check' feature.")
        parser.add_argument('-ext', '--extendedDisparity', action="store_true",
                            help="Enable stereo 'Extended Disparity' feature.")
        parser.add_argument('-sub', '--subpixel', action="store_true",
                            help="Enable stereo 'Subpixel' feature.")
        parser.add_argument('--debug', action="store_true",
                            help="Enables debug mode. Capability to connect to already BOOTED devices and also implicitly disables version check")

        self.args = parser.parse_args()

    @property
    def debug(self):
        return self.args.debug

    @property
    def max_disparity(self):
        max_disparity = 95
        if self.args.extendedDisparity:
            max_disparity *= 2
        if self.args.subpixel:
            max_disparity *= 32

        return max_disparity

    @staticmethod
    def ir_enabled(device):
        try:
            drivers = device.getIrDrivers()
            return len(drivers) > 0
        except RuntimeError:
            return False

    @staticmethod
    def rgb_resolution_shape(res: dai.ColorCameraProperties.SensorResolution) -> tuple[int, int]:
        if res == dai.ColorCameraProperties.SensorResolution.THE_720_P:
            return 1280, 720
        elif res == dai.ColorCameraProperties.SensorResolution.THE_800_P:
            return 1280, 800
        elif res == dai.ColorCameraProperties.SensorResolution.THE_1080_P:
            return 1920, 1080
        elif res == dai.ColorCameraProperties.SensorResolution.THE_4_K:
            return 3840, 2160
        elif res == dai.ColorCameraProperties.SensorResolution.THE_12_MP:
            return 4056, 3040
        elif res == dai.ColorCameraProperties.SensorResolution.THE_13_MP:
            return 4208, 3120
        else:
            raise Exception('Resolution not supported!')

    def get_resolution(self, device):
        cams = device.getConnectedCameras()

        sensorNames = device.getCameraSensorNames()
        if dai.CameraBoardSocket.RGB in cams:
            name = sensorNames[dai.CameraBoardSocket.RGB]
            if name == 'OV9782':
                res = dai.ColorCameraProperties.SensorResolution.THE_800_P
                cli_print(f'Camera sensor: {name}; resolution: {self.rgb_resolution_shape(res)}', PrintColors.RED)
                return res
            else:
                res = dai.ColorCameraProperties.SensorResolution.THE_1080_P
                cli_print(f'Camera sensor: {name}; resolution: {self.rgb_resolution_shape(res)}', PrintColors.RED)
                return res

    @staticmethod
    def is_usb2(device):
        device_info = device.getDeviceInfo()
        if device_info.protocol != dai.XLinkProtocol.X_LINK_USB_VSC:
            cli_print("Enabling low-bandwidth mode due to connection mode... (protocol: {})".format(device_info.protocol),
                      PrintColors.RED)
            return True
        if device.getUsbSpeed() not in [dai.UsbSpeed.SUPER, dai.UsbSpeed.SUPER_PLUS]:
            cli_print("Enabling low-bandwidth mode due to low USB speed... (speed: {})".format(device.getUsbSpeed()),
                      PrintColors.RED)
            return True

        return False

    @staticmethod
    def check_udev_rules():
        if platform.system() == 'Linux':
            ret = subprocess.call(['grep', '-irn', 'ATTRS{idVendor}=="03e7"', '/etc/udev/rules.d'],
                                  stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            if ret != 0:
                cli_print("WARNING: Usb rules not found", PrintColors.WARNING)
                cli_print("""
Run the following commands to set USB rules:

$ echo 'SUBSYSTEM=="usb", ATTRS{idVendor}=="03e7", MODE="0666"' | sudo tee /etc/udev/rules.d/80-movidius.rules
$ sudo udevadm control --reload-rules && sudo udevadm trigger

After executing these commands, disconnect and reconnect USB cable to your OAK device""", PrintColors.RED)
                raise SystemExit(1)

    @staticmethod
    def is_low_capabilities():
        return platform.machine().startswith("arm") or platform.machine().startswith("aarch")
